# 通常の KWIC
# ただし，KWIC のクラスをもつ

require 'jcode'
require 'kconv'

KWIC_RANK_STANDARD = 10

# KWIC のクラス定義
class KWIC < Object
  def initialize(line,first,last,num=nil,rank=KWIC_RANK_STANDARD)
    @line =  line         # 文1行
    @first = first        # 文中のキーワードの開始位置
    @last =  last         # 文中のキーワードの終了位置
    @num =  num           # この文が元のテキストの何行目か
    @rank =  rank         # ソートや表示で使用するランク
  end
  attr_reader :line, :first, :last, :num, :rank

  # KWIC のキーワードを返す．
  def key
    return @line[@first...@last]
  end

  # KWIC のキーワードの長さを返す．
  def key_length
    return @last-@first
  end

  # KWIC 形式でディスプレイへ表示
  def output(width,start_marker="",end_marker="")
    # width: テキストを表示する幅
    # start_marker,end_marker: キーワードの前後に置く記号

    len = @last-@first

    # 幅がキーワードの長さより狭いとき
    return start_marker + @line[@first,width] + end_marker if width < len

    left_margin = (width - len) / 2 / 2 * 2
    right_margin = width - len - left_margin
    
    if left_margin > @first
      w = " " * (left_margin - @first)
      left_edge = 0
    else
      w = ""
      left_edge = @first - left_margin
    end

    w = w + @line[left_edge...@first] + start_marker + @line[@first...@last] + end_marker
    right_edge = @last + right_margin
    if right_edge > @line.length
      w_end = " " * (right_edge - @line.length)
      right_edge = @line.length
    else
      w_end = ""
    end
    w = w + @line[@last...right_edge] + w_end     if @line[@last...right_edge]
    w
  end
end

# KWICの配列のためのクラス
class ArrayKWIC < Array
  def sorting(direction=false)
    # direction が false 以外なら，左方向へソート．
    # 但し，日本語では順序が正しくならない．
    # これは，reverse が日本語に対応していないから
    if !direction
      ary = self.sort{|x,y| x.line[x.first...-1].downcase <=> y.line[y.first...-1].downcase}
    else
      ary = self.sort{|x,y| x.line[0...x.first].reverse <=> y.line[0...y.first].reverse}
    end
  end
  
  def sorting!(direction=false)
    # direction が false 以外なら，左方向へソート．
    # 但し，日本語では順序が正しくならない．
    # これは，reverse が日本語に対応していないから
    if !direction
      ary = self.sort!{|x,y| x.line[x.first...-1].downcase <=> y.line[y.first...-1].downcase}
    else
      ary = self.sort!{|x,y| x.line[0...x.first].reverse <=> y.line[0...y.first].reverse}
    end
  end
  
  def max_first
    # キーワードの位置が一番右にくるときの値を返す．
    max = 0
    self.each{|l|
      max = l.first if max < l.first
    }
    max
  end

  def width
    # kwic 表示で一番右にくる文字の位置
    return 0 if self.empty?
    len = self[0].key_length
    left = self.max_first
    max = 0
    self.each{|l|
      right = l.line.length - l.last
      max = right if max < right
    }
#    left + len + max
    left + max
  end
end
