#! /usr/bin/env ruby

JAPANESE = 'Japanese'
ENGLISH = 'English'
FRENCH = 'French'
THAI = 'Thai'

require 'sary'
require 'koyori/bilingual_kwic'
require 'strscan'

$equiv_len = Array.new(2)
$language = Array.new(2)
$user_dic = Array.new(2)
$stop_word = Array.new(2)

$language[0] = JAPANESE
$language[1] = ENGLISH

$equiv_len[0] = 6
$equiv_len[1] = 4
$Japanese = /\xe3[\x81-\x83][\x80-\xbf]|[\xe4-\xe9][\x80-\xbf][\x80-\xbf]/u
$Japanese_pattern = /\xe3\x82[\xa1-\xbf]|\xe3\x83[\x80-\xb6]|[\xe4-\xe9][\x80-\xbf][\x80-\xbf]/u  # カタカナは含むが ひらがなは含まない．
$Thai = /\xe0[\xb8-\xb9][\x80-\xbf]/u   # 要検討

class Sary::Searcher
  def grep_num(pattern)
    # 同じ行に複数のキーワードが現れても，1回しか出力しない．
    # 返り値は行番号のソートされた配列と
    # 内容の入ったハッシュと出現回数
    text = Hash.new
    ary = Array.new

    if self.icase_search(pattern)
      count = self.count_occurrences
      self.sort_occurrences
      self.each_context_line{|line|
	line.sub!(/\A(\d+):/,'')
	num = $1.to_i
	ary << num
	text[num] = line
      }
      [ary,text,count]
    else
      nil
    end
  end
end

def set_Japanese_code(hiragana_search=false)
# 日本語の正規表現を指定
# hiragana_search の値により，平仮名を含むかどうかを変更する

  case $KCODE[0]
  when ?u, ?U
    # for uft-8
    $Japanese = /\xe3[\x81-\x83][\x80-\xbf]|[\xe4-\xe9][\x80-\xbf][\x80-\xbf]/u
    if hiragana_search
      $Japanese_pattern = /\xe3[\x81-\x83][\x80-\xbf]|[\xe4-\xe9][\x80-\xbf][\x80-\xbf]/u  # カタカナ・ひらがなも含む
    else
      $Japanese_pattern = /\xe3\x82[\xa1-\xbf]|\xe3\x83[\x80-\xb6]|[\xe4-\xe9][\x80-\xbf][\x80-\xbf]/u  # カタカナは含むが ひらがなは含まない．
    end
  else
    $Japanese = /\x83[\x40-\x96]|[\x88-\x9f\xe0-\xef][\x40-\x7e\x80-\xfc]|\x81\x5b|\x82[\x9F-\xf1]/s  # for SJIS
    if hiragana_search
      $Japanese_pattern = /\x82[\x9f-\xf1]|\x83[\x40-\x96]|[\x88-\x9f\xe0-\xef][\x40-\x7e\x80-\xfc]/s # for SJIS ひらがなも含む
    else
      $Japanese_pattern = /\x83[\x40-\x96]|[\x88-\x9f\xe0-\xef][\x40-\x7e\x80-\xfc]/s # for SJIS ひらがなは含まない
    end
  end
end

def del_header(line)
  return line.sub(/\A\d+:/,'')
end

def character_multigram(filename,pat=/\w/,max=6,area2=nil)
  ngram = Hash.new(0)
  tmp_gram = Hash.new
  word = Array.new(max)

  if area2   # area2 を Hash に変換．ただし．area2 がないときは常に真
    area = Hash.new
    area2.each{|a| area[a] = true}
  else
    area = Hash.new(true)
  end

  File.open(filename){|file|
    file.each{|line|
      next unless area[file.lineno]
      line = del_header(line)
      word.clear
      tmp_gram.clear
      s = StringScanner.new(line) # line.scan(/./){|w1| より少し速い
      while !s.eos?
	word[max-1] = s.getch
	phrase = ''
	word.reverse.each{|w|
	  break if !w || (w !~ pat)
	  phrase = w + phrase
	  tmp_gram[phrase] = true
	}
	word.shift
      end
      tmp_gram.each_key{|k| ngram[k] += 1 }
    }
  }
  ngram
end

def word_unigram(filename,pat,area2=nil)
  unigram = Hash.new(0)

  if area2   # area2 を Hash に変換．ただし．area2 がないときは常に真
    area = Hash.new
    area2.each{|a| area[a] = true}
  else
    area = Hash.new(true)
  end

  File.open(filename){|file|
    file.each{|line|
      next unless area[file.lineno]
      line.scan(pat){|w|
	unigram[w.downcase] += 1
      }
    }
  }
  unigram
end

def word_multigram(filename,pat=/\w+/,max=4,area2=nil,noncandidate={"a"=>true,"an"=>true,"the"=>true})
  # 単語 n-gram をまとめて計算する．
  # ただし，1行に複数回出現しても1回としか数えない
  # max は最大長
  # 配列 area2 があるときは，その行番号だけを検索対象とする
  # noncandidate は Hash で，値が true となる物が含まれる n-gram は計算しない
  ngram = Hash.new(0)
  tmp_gram = Hash.new
  word = Array.new(max)

  if area2   # area2 を Hash に変換．ただし．area2 がないときは常に真
    area = Hash.new
    area2.each{|a| area[a] = true}
  else
    area = Hash.new(true)
  end

  File.open(filename){|file|
    file.each{|line|
      next unless area[file.lineno]
      word.clear
      tmp_gram.clear
      line.scan(pat){|w1|
	word[max-1] = w1.downcase
	phrase = ''
	word.reverse.each{|w|
	  break if !w || noncandidate[w]
	  phrase = w + phrase
	  tmp_gram[phrase] = true
	  phrase = ' ' + phrase
	}
	word.shift
      }
      tmp_gram.each_key{|k| ngram[k] += 1 }
    }
  }
  ngram
end

def word_alignment(key,ngram,s_searcher,t_searcher,stop_word=[])
  # word alignment を行う関数
  # key:   原言語のコーパスに含まれるキーワード
  # ngram: 対象言語の特定領域から収集した対訳候補の n-gram
  # s_searcher: 原言語の Suffix Array
  # t_searcher: 対象言語の Suffix Array

  stop_word.each{|w| ngram.delete(w)}

  p ngram  if $KOYORI_DEBUG

  if s_searcher.icase_search(key)
    freq_source = s_searcher.count_occurrences

    # 閾値
    printf("key = %s,freq_source = %d\n",key,freq_source) if $KOYORI_DEBUG
    theta = freq_source / 10
    theta = 1  if theta < 1
#    theta = 5

    equivalents = ngram.keys.collect{|word|
      freq_co_occur = ngram[word]
      printf("word = %s,freq_co_occur = %d\n",word,freq_co_occur) if $KOYORI_DEBUG
      next if freq_co_occur < theta
      if t_searcher.icase_search(word)
	freq_target = t_searcher.count_occurrences
	# 英語の場合，接頭語として含まれる物も数えている．
	# これは名詞の複数形や動詞の活用形に対処するため．
      else
#	print "warning:",word," is not found in the target corpus\n"
	## 単語間にコンマなどがあると，0になる可能性がある．
#	freq_target = 0
	next
      end
      sim = 2.0 * freq_co_occur / (freq_source + freq_target)
      [sim,word,freq_target]
    }.compact.sort_by{|x| [-x[0],-x[1].size]}
  else
    return Array.new
  end
  
  equivalents
end

def make_bilingual_kwic(key,t_key,files,noncandidate,kwd_lang)
# Bilingual KWIC を作る関数
# ここの noncandidate は word n-gram を作成するときに利用
# t_key は 対訳語．これがある場合には，まず対訳語を指定した KWIC を作る．

  kwic_ary = ArrayBilingualKWIC.new
  candidates = Array.new

#  sl : source language number

  if !kwd_lang
    # キーワードの自動判定
    if $language[0] == JAPANESE
      if key =~ $Japanese
	sl = 0; tl = 1
      else
	sl = 1; tl = 0
      end
    elsif $language[1] == JAPANESE
      if key =~ $Japanese
	sl = 1; tl = 0
      else
	sl = 0; tl = 1
      end
    elsif $language[0] == THAI
      if key =~ $Thai
	sl = 0; tl = 1
      else
	sl = 1; tl = 0
      end
    elsif $language[1] == THAI
      if key =~ $Thai
	sl = 1; tl = 0
      else
	sl = 0; tl = 1
      end
    end
  else
    if kwd_lang == $language[0]
      sl = 0; tl = 1
    else
      sl = 1; tl = 0
    end
  end

  s_searcher = Sary::Searcher.new(files[sl])
  t_searcher = Sary::Searcher.new(files[tl])
  s_file = files[sl]
  t_file = files[tl]
  s_len = $equiv_len[sl]
  t_len = $equiv_len[tl]
  user_dic = $user_dic[sl]
  
  area,s_text,s_count = s_searcher.grep_num(key)
  return [kwic_ary,candidates,0,sl,tl] if !area

  if t_key  # 対訳語が指定された kwic を作成する
    rank = 0
    kwic_ary,area,t_count,del = add_bilingual_kwic(kwic_ary,area,key,t_key,s_text,t_searcher,sl,tl,rank)
    candidates << [1.0,t_key,t_count]
  end

  if $use_dictionary && user_dic[key]
    rank = 1  # 辞書にある単語は 1 ≦ rank < KWIC_RANK_STANDARD
    user_dic[key].split(';').each{|t_key|
      t_key.sub!(/\#.*\z/,'')  # コメントの削除
      kwic_ary,area,t_count,del = add_bilingual_kwic(kwic_ary,area,key,t_key,s_text,t_searcher,sl,tl,rank)
      candidates << [1.0,t_key,t_count]
      rank = rank + 1
    }
  end

  rank = KWIC_RANK_STANDARD # 単純に rank の値を +1 しないのは表示色のため
  while area.size > 1
    case $language[tl]
    when JAPANESE
      ngram = character_multigram(t_file,$Japanese_pattern,t_len,area)
    when THAI
      ngram = character_multigram(t_file,/\w/,t_len,area)
    else
      ngram = word_multigram(t_file,/\w+/,t_len,area,noncandidate[tl])
    end

    tmp_candidates = word_alignment(key,ngram,s_searcher,t_searcher,$stop_word[tl])

    if tmp_candidates[0] && tmp_candidates[0][1]
      candidates = candidates + tmp_candidates
      equivalent = tmp_candidates[0][1]
      kwic_ary,area,t_count,del = add_bilingual_kwic(kwic_ary,area,key,equivalent,s_text,t_searcher,sl,tl,rank)
      rank = rank + 1
      break if del < 1
    else
      break
    end
  end

  # 対訳語が見つからなかった行を追加する
  File.open(t_file,"r").each{|line|
    line.sub!(/^(\d+):/,'')
    num = Regexp.last_match(1).to_i
    if num == area[0]
      s_line = s_text[num].chomp
      s_first = s_line.index(/#{key}/i)
      s_last = s_first + Regexp.last_match(0).size
      kwic = BilingualKWIC.new(s_line,s_first,s_last,line.chomp,0,0,rank)
#      rank = rank + 1 # 見つからなかったものは，みな同じ rank

      kwic_ary << kwic
      area.shift
      break if area.empty?
    end
  }

  [kwic_ary,candidates,s_count,sl,tl]
end

def set_noncandidate(lang)
=begin
  下の旧バージョンと違い，単純に冠詞を検索候補から外す．
=end
  noncandidate = Hash.new
  
  case lang
  when ENGLISH
    noncandidate["a"] = true
    noncandidate["an"] = true
    noncandidate["the"] = true
  else FRENCH
    noncandidate["le"] = true
    noncandidate["la"] = true
    noncandidate["l"] = true
    noncandidate["les"] = true
    noncandidate["un"] = true
    noncandidate["une"] = true
    noncandidate["des"] = true
    noncandidate["du"] = true
    noncandidate["de"] = true
  end
  
  noncandidate
end

def add_bilingual_kwic(kwic_ary,area,s_key,t_key,s_text,t_searcher,sl,tl,score=10)
=begin
 特定の area 内で，キーワードと対訳語を指定して Bilingual KWIC を作成する．
 sl: source language number
 tl: target language number
返り値は，Bilingual KWIC と area 
=end

  case $language[tl]
  when JAPANESE,THAI
    result = t_searcher.search(t_key)
  else
    result = t_searcher.icase_search(t_key)
  end
  occurrence = 0

  if result
    t_searcher.sort_occurrences
    occurrence = t_searcher.count_occurrences
    found = []
    t_searcher.each_context_line {|t_text|
      line = t_text.sub(/^\d+:/,'')
      line_num = Regexp.last_match(0).to_i
      if s_text[line_num]
	s_line = s_text[line_num].chomp
	t_line = line.chomp
	
	case $language[sl]
	when JAPANESE,THAI
	  s_pattern = /#{s_key}/
	else
	  s_pattern = /\b#{s_key}/i
	end
	
	case $language[tl]
	when JAPANESE,THAI
	  t_pattern = /#{t_key}/
	else
	  t_pattern = /\b#{t_key}/i
	end
	
	# 1行にキーワードが複数回出現するのに対応した処理
	s_last = 0
	if $title && /^\[(.+?)\]/ =~ s_line  # タイトルがあった場合の処理
	  s_last = $&.size
	end
	while s_first = s_line.index(s_pattern,s_last)
	  s_last = s_first + Regexp.last_match(0).size
	  t_last = 0
	  if $title && /^\[(.+?)\]/ =~ t_line  # タイトルがあった場合の処理
	    t_last = $&.size
	  end
	  while t_first = t_line.index(t_pattern,t_last)
	    t_last = t_first + Regexp.last_match(0).size
	    kwic = BilingualKWIC.new(s_line,s_first,s_last,
				     t_line,t_first,t_last,score)
	    kwic_ary << kwic
	    break unless $verbose_candidates
	  end
	  break unless $verbose_candidates
	end
	# 候補が見つかった行は削除．
	s_text[line_num] = nil #unless $verbose_candidates
	found << line_num
      end
    }
    area = area - found
    deleted_num = found.size
  end
  [kwic_ary,area,occurrence,deleted_num]
end

